// Lawson Labs, Inc.
// 3217 Phoenixville Pike
// Malvern, PA  19355
// 800 321-5355 or 610 725-8800             
// 9-28-98
// filename: NT_m201ax.h

//typedef for M201 struct
//typedef struct {
struct T_M201{
	short		comPort;			//com port number 1-4
	double	scanInterval;	//scan interval in seconds
	short		numChannels;	//number of active channels for scanning
	short		tScan;			//type of scan 0 = single channel 1 = normal 2 = normal with calibration
	short		chan[6];			//control code array for scanning
	short		singleChan;		//single channel scanning channel
	short		lastDigInp; 	//last digital input value
	short 	version;			//Model 201 version
	short 	channel;			//currently selected channel (used in GetConversion)
	short 	subChan;			//currently selected sub channel (used in GetConversion)
	long		rawCount;		//current raw count returned from Model 201
	double	voltage;			//current voltage
	int		digOutWord;		//value sent to Model 201 for digital output (0-255)
	short 	wordCnt;			//number of words in conversion (2 or 3)
	short 	biPolar;			//0 = unipolar  1 = bipolar
	int		errorType;		//error code returned from DLL
	short 	filter;			//0 = 4 Hz  1 = 40 Hz  2 = 400 Hz
	short 	gain;				//gain factor (powers of 2)
	float 	rate;				//sample rate (10 - 1027 Hz)
	short 	modeRegLo;		//mode registers for signon
	short 	modeRegMid;
	short 	modeRegHi;
	short 	baud;				//selected baud rate code (0 - 5)
	short 	sleeping;		//0 = Model 201 awake  1 = Model 201 asleep
	short 	standBy;			//0 = Model 201 out of standby  1 = Model 201 in standby
	int		timeBaseCnt;	//mode 202 specific element (reserved)
	unsigned int    average; //remote averaging (powers of 2) //transmit/receive queue sizes as used by Windows
	unsigned int    TXQUEUESIZE;
	unsigned int    RXQUEUESIZE;
	BYTE NewScanBuff[128];	
	BYTE cbuf[25];
	short checkSum;
	BYTE last_char;
	UINT       ConvWait;//calculated conversion wait time
	COMSTAT portStat;	  // structure for comm status
	BYTE fDoLabel;
	DCB dcb;  //  com port control structure used by Windows
	int buffer_size;  //  current size of buffer
	BYTE fFirstDiginRequested;
}; 

//BYTE NewScanBuff[128];
//BYTE *pNewScanBuff;		

double maxTime, minTime;

//local prototypes

BOOLEAN LoadLibraries(void);
BOOLEAN UnloadLibraries(void);
BOOLEAN CheckLibFuncs(void);

//turns on cursor in Win32 console
void    SetWin32CursorOn(void);

//turns off cursor in Win32 console
void    SetWin32CursorOff(void);

//read 201EWIN.CFG file from disk in current directory
short   ReadConfig(short *localAvg);

//write 201EWIN.CFG file to disk in current directory
short   WriteConfig(short *localAvg);

//select the scan interval
short   CalcScanTime(void);

//displays the scanning menu (returns 1 if set to start scanning)
short   ScanMenu(void);

//select the sampling rate for the A/D
void    SelectRate(void);

// Get starting & ending control codes for a scanning channel
short   GetChannel(int value);

//display polled mode functions menu
void    PrintCommandMenu(int Ch, short localAvg, short checkSum);

//perform selected function
void    DoCommand(int Ch, double *max, double *min,
						short *localAvg, short *checkSum);

//calculate standard deviation
void    StandardDev(double *stDv, double *rmsBits, double *sdTot,
						  double *runningAvg, double *sd, int *sdCount);

//=======================================\
//prototypes for external dll functions
//typedef short(*DLL_SET_PTRS)(BYTE *pNewScanBuff);
//DLL_SET_PTRS SetPtrs;

//3-11-02
//12-5-02 typedef UINT(*DLL_GET_SCAN_PNT_CNT)(void);
//12-5-02 DLL_GET_SCAN_PNT_CNT GetNumScansInBuffer;
//12-5-02 typedef UINT(*DLL_GET_SCAN_BYTE_CNT)(void);
//12-5-02 DLL_GET_SCAN_BYTE_CNT GetNumBytesInBuffer;


typedef UINT(*DLL_GET_SCAN_PNT_CNT)(T_M201 *pM201);
DLL_GET_SCAN_PNT_CNT GetNumScansInBuffer;
typedef UINT(*DLL_GET_SCAN_BYTE_CNT)(T_M201 *pM201);
DLL_GET_SCAN_BYTE_CNT GetNumBytesInBuffer;




//This function will sign on to the Model 201 using the settings in M201
//If could not sign on, closes port and returns 0. Otherwise, returns 1,
//port is open and the Model 201 is active.

//typedef short(*DLL_SIGN_ON)(void);
typedef short(*DLL_SIGN_ON)(T_M201 *pM201);
DLL_SIGN_ON SignOn;

//take a raw count and scale in volts
//typedef double(*DLL_COUNTS_2_VOLTS)(void);
typedef double(*DLL_COUNTS_2_VOLTS)(T_M201 *pM201);
DLL_COUNTS_2_VOLTS Counts2Volts;

//Get a digital input from the Model 201.  Returns 0 if Model 201 did not
//echo back the Get Digital Input token.
//typedef short(*DLL_GET_DIG_IN)(void);
typedef short(*DLL_GET_DIG_IN)(T_M201 *pM201);
DLL_GET_DIG_IN GetDigIn;

//Get the Model 201 version.  Returns 0 if Model 201 did not echo back the
//version token.
//typedef short(*DLL_GET_VER)(void);
typedef short(*DLL_GET_VER)(T_M201 *pM201);
DLL_GET_VER GetVer;

//Send a digital output to the Model 201
//typedef short(*DLL_SET_DIG_OUT)(int digOutWord);
typedef short(*DLL_SET_DIG_OUT)(T_M201 *pM201,int digOutWord);
DLL_SET_DIG_OUT SetDigOut; 

//Set averaging in the Model 201 (powers of 2)
//typedef short(*DLL_REMOTE_AVG)(void);
typedef short(*DLL_REMOTE_AVG)(T_M201 *pM201);
DLL_REMOTE_AVG RemoteAvg;

//Perform a full scale calibration on the currently selected channel
//typedef short(*DLL_FULL_SCALE_CAL)(void);
typedef short(*DLL_FULL_SCALE_CAL)(T_M201 *pM201);
DLL_FULL_SCALE_CAL FullScaleCal; 

//Perform an offset calibration on the currently selected channel
//typedef short(*DLL_OFFSET_CAL)(void);
typedef short(*DLL_OFFSET_CAL)(T_M201 *pM201);
DLL_OFFSET_CAL OffsetCal;

//Perform a system calibration (full scale on channel 6, offset on channel 7)
//typedef short(*DLL_SYS_CAL)(void);
typedef short(*DLL_SYS_CAL)(T_M201 *pM201);
DLL_SYS_CAL SysCal;

//Get a conversion from the Model 201 on the currently selected channel
//typedef short(*DLL_GET_CONVERSION)(void);
typedef short(*DLL_GET_CONVERSION)(T_M201 *pM201);
DLL_GET_CONVERSION GetConversion;

//Select channel and subchannel from M201.channel and M201.subChan
//short  (FAR PASCAL *SetChanCode)(T_M201 *M201);
//typedef short(*DLL_SET_CHAN_CODE)(void);
typedef short(*DLL_SET_CHAN_CODE)(T_M201 *pM201);
DLL_SET_CHAN_CODE SetChanCode;

//Start scanning using selected scan type. Returns 0 if scan token is not
//echoed back.
//typedef short(*DLL_START_SCANNING)(void);
typedef short(*DLL_START_SCANNING)(T_M201 *pM201);
DLL_START_SCANNING StartScanning;

//Get one scan from currently selected comm port buffer. It will wait the
//calculated time for the scan to come in. Returns 0 if failed. Returns type
//of failure in M201.errorType.
//M201.errorType = 1		Start scan framing failure
//M201.errorType = 2		not enough chars, possible communications error
//M201.errorType = 3		Comm port buffer overflow
//M201.errorType = 0xA  Scan rate too fast for this scan type
//short  (FAR PASCAL *GetOneScan)(T_M201 *M201, long *clientDataBuffer);
//typedef short(*DLL_GET_ONE_SCAN)(void);
typedef short(*DLL_GET_ONE_SCAN)(T_M201 *pM201);
DLL_GET_ONE_SCAN GetOneScan;

//Tell the Model 201 to stop scanning. Returns 0 if Stop scan token is not
//echoed back
//typedef short(*DLL_STOP_SCANNING)(void);
typedef short(*DLL_STOP_SCANNING)(T_M201 *pM201);
DLL_STOP_SCANNING StopScanning;

//Get counts from ScanBuffer, scale them in volts, and remove counts from
//ScanBuffer
//typedef void(*DLL_GCONV)(void);
typedef void(*DLL_GCONV)(T_M201 *pM201);
DLL_GCONV Gconv;

//Remove size elements from ScanBuffer
//typedef void(*DLL_REMOVE_FROM_BUFFER)(short size);
typedef void(*DLL_REMOVE_FROM_BUFFER)(T_M201 *pM201,short size);
DLL_REMOVE_FROM_BUFFER RemFromBuffer;

//Set the filter  0 = 4 Hz   1 = 40 Hz   2 = 400 Hz
//typedef void(*DLL_SET_FILTER)(short filter);
typedef void(*DLL_SET_FILTER)(T_M201 *pM201);
DLL_SET_FILTER SetFilter;

//Send mode registers to Model 201. Returns 0 if the registers read back
//incorrectly
//typedef short(*DLL_SEND_AD_MODE)(void);
typedef short(*DLL_SEND_AD_MODE)(T_M201 *pM201);
DLL_SEND_AD_MODE SendADMode;


//Get and compare Model 201 checksum. Returns 0 if failed, 1 if checksum is OK
//typedef short(*DLL_GET_CHECK_SUM)(void);
typedef short(*DLL_GET_CHECK_SUM)(T_M201 *pM201);
DLL_GET_CHECK_SUM GetCheckSum;

//Send the stop scan command to stop a scan if scanning, send master reset,
//and close the currently selected comm port.
//typedef short(*DLL_CLOSE_DEVICE)(void);
typedef short(*DLL_CLOSE_DEVICE)(T_M201 *pM201);
DLL_CLOSE_DEVICE CloseDevice; 

//Put the Model 201 in sleep mode
//typedef short(*DLL_GOTO_SLEEP)(void);
typedef short(*DLL_GOTO_SLEEP)(T_M201 *pM201);
DLL_GOTO_SLEEP GotoSleep;

//Wait the filter settling time
//typedef void(*DLL_FILTER_DELAY)(short filter);
typedef void(*DLL_FILTER_DELAY)(T_M201 *pM201);
DLL_FILTER_DELAY FilterDelay;

//Check the selected scan interval to see if it is in limits
//typedef short(*DLL_CHECK_SCAN_TIME)(double *maxTime, double *minTime);
typedef short(*DLL_CHECK_SCAN_TIME)(T_M201 *pM201,double *maxTime, double *minTime);
DLL_CHECK_SCAN_TIME CheckScanTime; 

//The next two functions only need to be called if handling more than one
//Model 201 at a time.

//Get the comm handle after Signon
typedef void(*DLL_GET_COM_HANDLE)(T_M201 *pM201,HANDLE *IdComDev);
DLL_GET_COM_HANDLE GetComHandle;

//Global Variables
HINSTANCE 	hLibrary;

// tim static HANDLE idComDev;   //  device channel to com port as used by Windows
HANDLE IdComDev;   //  device channel to com port as used by Windows
